#include "G4HumanPhantomSD.h"

#include "G4HCofThisEvent.hh"
#include "G4SDManager.hh"
#include "G4Step.hh"
#include "G4ios.hh"

G4HumanPhantomSD::G4HumanPhantomSD(const G4String& name, const G4String& hitsCollectionName)
    : G4VSensitiveDetector(name) {
    collectionName.insert(hitsCollectionName);
}

G4HumanPhantomSD::~G4HumanPhantomSD() {}

void G4HumanPhantomSD::Initialize(G4HCofThisEvent* HCE) {
    collection = new G4HumanPhantomHitsCollection(SensitiveDetectorName, collectionName[0]);
    static G4int HCID = -1;
    if (HCID < 0) {
        HCID = G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]);
    }
    HCE->AddHitsCollection(HCID, collection);
}

G4bool G4HumanPhantomSD::ProcessHits(G4Step* aStep, G4TouchableHistory*) {
    G4double edep = aStep->GetTotalEnergyDeposit();

    if (edep == 0.) return false;

    G4String bodypartName = aStep->GetPreStepPoint()->GetTouchable()->GetVolume()->GetLogicalVolume()->GetName();

    G4HumanPhantomHit* newHit = new G4HumanPhantomHit();
    newHit->SetEdep(edep);
    newHit->SetBodyPartID(bodypartName);
    collection->insert(newHit);

    return true;
}

void G4HumanPhantomSD::EndOfEvent(G4HCofThisEvent*) {}
