#include "G4HumanPhantomColour.h"
#include "G4HumanPhantomMaterial.h"
#include "G4MIRDLeftClavicle.h"

#include "G4Box.hh"
#include "G4Cons.hh"
#include "G4EllipticalTube.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4RotationMatrix.hh"
#include "G4SDManager.hh"
#include "G4SubtractionSolid.hh"
#include "G4SystemOfUnits.hh"
#include "G4ThreeVector.hh"
#include "G4Torus.hh"
#include "G4VPhysicalVolume.hh"
#include "G4VisAttributes.hh"
#include "globals.hh"

G4MIRDLeftClavicle::G4MIRDLeftClavicle() {}

G4MIRDLeftClavicle::~G4MIRDLeftClavicle() {}

G4VPhysicalVolume* G4MIRDLeftClavicle::Construct(const G4String& volumeName, G4VPhysicalVolume* mother,
                                                 const G4String& colourName, G4bool wireFrame, G4bool) {
    G4cout << "Construct " << volumeName << " with mother volume " << mother->GetName() << G4endl;

    G4HumanPhantomMaterial* material = new G4HumanPhantomMaterial();
    G4Material* skeleton = material->GetMaterial("skeleton");

    G4double rMin = 0 * cm;
    G4double rMax = 0.7883 * cm;
    G4double rTor = 10 * cm;
    G4double pSPhi = 298.15 * degree;
    G4double pDPhi = 0.7 * rad;

    G4Torus* clavicle = new G4Torus("Clavicle", rMin, rMax, rTor, pSPhi, pDPhi);

    G4LogicalVolume* logicLeftClavicle = new G4LogicalVolume(clavicle, skeleton, "logical" + volumeName, 0, 0, 0);

    G4VPhysicalVolume* physLeftClavicle =
        new G4PVPlacement(0, G4ThreeVector(0. * cm, 2. * cm, 33.25 * cm), "physicalLeftClavicle", logicLeftClavicle,
                          mother, false, 0, true);

    // Visualization Attributes
    // G4VisAttributes* LeftClavicleVisAtt = new G4VisAttributes(G4Colour(0.94,0.5,0.5));
    G4HumanPhantomColour* colourPointer = new G4HumanPhantomColour();
    G4Colour colour = colourPointer->GetColour(colourName);
    G4VisAttributes* LeftClavicleVisAtt = new G4VisAttributes(colour);
    LeftClavicleVisAtt->SetForceSolid(wireFrame);
    logicLeftClavicle->SetVisAttributes(LeftClavicleVisAtt);

    G4cout << "LeftClavicle created !!!!!!" << G4endl;

    // Testing LeftClavicle Volume
    G4double LeftClavicleVol = logicLeftClavicle->GetSolid()->GetCubicVolume();
    G4cout << "Volume of LeftClavicle = " << LeftClavicleVol / cm3 << " cm^3" << G4endl;

    // Testing LeftClavicle Material
    G4String LeftClavicleMat = logicLeftClavicle->GetMaterial()->GetName();
    G4cout << "Material of LeftClavicle = " << LeftClavicleMat << G4endl;

    // Testing Density
    G4double LeftClavicleDensity = logicLeftClavicle->GetMaterial()->GetDensity();
    G4cout << "Density of Material = " << LeftClavicleDensity * cm3 / g << " g/cm^3" << G4endl;

    // Testing Mass
    G4double LeftClavicleMass = (LeftClavicleVol)*LeftClavicleDensity;
    G4cout << "Mass of LeftClavicle = " << LeftClavicleMass / gram << " g" << G4endl;

    return physLeftClavicle;
}
