#include "G4HumanPhantomColour.h"
#include "G4HumanPhantomMaterial.h"
#include "G4MIRDLeftLegBone.h"

#include "G4Cons.hh"
#include "G4EllipticalTube.hh"
#include "G4PVPlacement.hh"
#include "G4RotationMatrix.hh"
#include "G4SDManager.hh"
#include "G4SystemOfUnits.hh"
#include "G4ThreeVector.hh"
#include "G4UnionSolid.hh"
#include "G4VPhysicalVolume.hh"
#include "G4VisAttributes.hh"
#include "globals.hh"

G4MIRDLeftLegBone::G4MIRDLeftLegBone() {}

G4MIRDLeftLegBone::~G4MIRDLeftLegBone() {}

G4VPhysicalVolume* G4MIRDLeftLegBone::Construct(const G4String& volumeName, G4VPhysicalVolume* mother,
                                                const G4String& colourName, G4bool wireFrame, G4bool) {
    G4HumanPhantomMaterial* material = new G4HumanPhantomMaterial();

    G4cout << "Construct " << volumeName << " with mother volume " << mother->GetName() << G4endl;

    G4Material* skeleton = material->GetMaterial("skeleton");

    delete material;

    G4double dz = 79.8 * cm;
    G4double rmin1 = 0.0 * cm;
    G4double rmin2 = 0.0 * cm;
    G4double rmax1 = 1. * cm;
    G4double rmax2 = 3.5 * cm;
    G4double startphi = 0. * degree;
    G4double deltaphi = 360. * degree;

    G4Cons* leg_bone = new G4Cons("OneLeftLegBone", rmin1, rmax1, rmin2, rmax2, dz / 2., startphi, deltaphi);

    G4LogicalVolume* logicLeftLegBone = new G4LogicalVolume(leg_bone, skeleton, "logical" + volumeName, 0, 0, 0);

    // Define rotation and position here!
    G4VPhysicalVolume* physLeftLegBone = new G4PVPlacement(
        0, G4ThreeVector(0.0 * cm, 0.0, 0.1 * cm), "physicalLeftLegBone", logicLeftLegBone, mother, false, 0, true);

    // Visualization Attributes
    // G4VisAttributes* LeftLegBoneVisAtt = new G4VisAttributes(G4Colour(0.46,0.53,0.6));
    G4HumanPhantomColour* colourPointer = new G4HumanPhantomColour();
    G4Colour colour = colourPointer->GetColour(colourName);
    G4VisAttributes* LeftLegBoneVisAtt = new G4VisAttributes(colour);

    LeftLegBoneVisAtt->SetForceSolid(wireFrame);
    logicLeftLegBone->SetVisAttributes(LeftLegBoneVisAtt);

    G4cout << "LeftLegBone created !!!!!!" << G4endl;

    // Testing LeftLegBone Volume
    G4double LeftLegBoneVol = logicLeftLegBone->GetSolid()->GetCubicVolume();
    G4cout << "Volume of LeftLegBone = " << LeftLegBoneVol / cm3 << " cm^3" << G4endl;

    // Testing LeftLegBone Material
    G4String LeftLegBoneMat = logicLeftLegBone->GetMaterial()->GetName();
    G4cout << "Material of LeftLegBone = " << LeftLegBoneMat << G4endl;

    // Testing Density
    G4double LeftLegBoneDensity = logicLeftLegBone->GetMaterial()->GetDensity();
    G4cout << "Density of Material = " << LeftLegBoneDensity * cm3 / g << " g/cm^3" << G4endl;

    // Testing Mass
    G4double LeftLegBoneMass = (LeftLegBoneVol)*LeftLegBoneDensity;
    G4cout << "Mass of LeftLegBone = " << LeftLegBoneMass / gram << " g" << G4endl;

    return physLeftLegBone;
}
