#include "G4HumanPhantomColour.h"
#include "G4HumanPhantomMaterial.h"
#include "G4MIRDLeftOvary.h"

#include "G4Ellipsoid.hh"
#include "G4LogicalVolume.hh"
#include "G4Material.hh"
#include "G4PVPlacement.hh"
#include "G4RotationMatrix.hh"
#include "G4SDManager.hh"
#include "G4SystemOfUnits.hh"
#include "G4ThreeVector.hh"
#include "G4UnionSolid.hh"
#include "G4VPhysicalVolume.hh"
#include "G4VisAttributes.hh"
#include "globals.hh"

G4MIRDLeftOvary::G4MIRDLeftOvary() {}

G4MIRDLeftOvary::~G4MIRDLeftOvary() {}

G4VPhysicalVolume* G4MIRDLeftOvary::Construct(const G4String& volumeName, G4VPhysicalVolume* mother,
                                              const G4String& colourName, G4bool wireFrame, G4bool) {
    G4cout << "Construct " << volumeName << " with mother volume " << mother->GetName() << G4endl;

    G4HumanPhantomMaterial* material = new G4HumanPhantomMaterial();
    G4Material* soft = material->GetMaterial("soft_tissue");
    delete material;

    G4double ax = 1. * cm;
    G4double by = 0.5 * cm;
    G4double cz = 2. * cm;

    G4Ellipsoid* OneOvary = new G4Ellipsoid("OneOvary", ax, by, cz);

    G4LogicalVolume* logicLeftOvary = new G4LogicalVolume(OneOvary, soft, "logical" + volumeName, 0, 0, 0);

    // Define rotation and position here!
    G4VPhysicalVolume* physLeftOvary = new G4PVPlacement(0, G4ThreeVector(-6. * cm, 0.5 * cm, -20 * cm),
                                                         "physicalLeftOvary", logicLeftOvary, mother, false, 0, true);

    // Visualization Attributes
    // G4VisAttributes* LeftOvaryVisAtt = new G4VisAttributes(G4Colour(0.85,0.44,0.84));
    G4HumanPhantomColour* colourPointer = new G4HumanPhantomColour();
    G4Colour colour = colourPointer->GetColour(colourName);
    delete colourPointer;

    G4VisAttributes* LeftOvaryVisAtt = new G4VisAttributes(colour);
    LeftOvaryVisAtt->SetForceSolid(wireFrame);
    logicLeftOvary->SetVisAttributes(LeftOvaryVisAtt);

    G4cout << "LeftOvary created !!!!!!" << G4endl;

    // Testing LeftOvary Volume
    G4double LeftOvaryVol = logicLeftOvary->GetSolid()->GetCubicVolume();
    G4cout << "Volume of LeftOvary = " << LeftOvaryVol / cm3 << " cm^3" << G4endl;

    // Testing LeftOvary Material
    G4String LeftOvaryMat = logicLeftOvary->GetMaterial()->GetName();
    G4cout << "Material of LeftOvary = " << LeftOvaryMat << G4endl;

    // Testing Density
    G4double LeftOvaryDensity = logicLeftOvary->GetMaterial()->GetDensity();
    G4cout << "Density of Material = " << LeftOvaryDensity * cm3 / g << " g/cm^3" << G4endl;

    // Testing Mass
    G4double LeftOvaryMass = (LeftOvaryVol)*LeftOvaryDensity;
    G4cout << "Mass of LeftOvary = " << LeftOvaryMass / gram << " g" << G4endl;

    return physLeftOvary;
}
