#include "G4HumanPhantomColour.h"
#include "G4HumanPhantomMaterial.h"
#include "G4MIRDSkull.h"

#include "G4Box.hh"
#include "G4Ellipsoid.hh"
#include "G4EllipticalTube.hh"
#include "G4PVPlacement.hh"
#include "G4RotationMatrix.hh"
#include "G4SDManager.hh"
#include "G4SubtractionSolid.hh"
#include "G4SystemOfUnits.hh"
#include "G4ThreeVector.hh"
#include "G4UnionSolid.hh"
#include "G4VPhysicalVolume.hh"
#include "G4VSolid.hh"
#include "G4VisAttributes.hh"
#include "globals.hh"

G4MIRDSkull::G4MIRDSkull() {}

G4MIRDSkull::~G4MIRDSkull() {}

G4VPhysicalVolume* G4MIRDSkull::Construct(const G4String& volumeName, G4VPhysicalVolume* mother,
                                          const G4String& colourName, G4bool wireFrame, G4bool) {
    G4HumanPhantomMaterial* material = new G4HumanPhantomMaterial();

    G4cout << "Construct " << volumeName << " with mother volume " << mother->GetName() << G4endl;

    G4Material* skeleton = material->GetMaterial("skeleton");

    delete material;

    // Outer cranium
    G4double ax = 6.8 * cm;  // a out skull
    G4double by = 9.8 * cm;  // bout
    G4double cz = 8.3 * cm;  // cout

    G4Ellipsoid* craniumOut = new G4Ellipsoid("CraniumOut", ax, by, cz);

    ax = 6. * cm;   // a in
    by = 9. * cm;   // b in
    cz = 6.5 * cm;  // cin

    G4Ellipsoid* craniumIn = new G4Ellipsoid("CraniumIn", ax, by, cz);

    G4SubtractionSolid* cranium =
        new G4SubtractionSolid("Cranium", craniumOut, craniumIn, 0, G4ThreeVector(0.0, 0.0, 1. * cm));

    G4LogicalVolume* logicSkull = new G4LogicalVolume(cranium, skeleton, "logical" + volumeName, 0, 0, 0);

    // Define rotation and position here!
    G4VPhysicalVolume* physSkull =
        new G4PVPlacement(0, G4ThreeVector(0., 0., 7.75 * cm), "physicalSkull", logicSkull, mother, false, 0, true);

    // Visualization Attributes
    // G4VisAttributes* SkullVisAtt = new G4VisAttributes(G4Colour(0.46,0.53,0.6));
    G4HumanPhantomColour* colourPointer = new G4HumanPhantomColour();
    G4Colour colour = colourPointer->GetColour(colourName);
    G4VisAttributes* SkullVisAtt = new G4VisAttributes(colour);
    SkullVisAtt->SetForceSolid(wireFrame);
    SkullVisAtt->SetLineWidth(4. * mm);
    logicSkull->SetVisAttributes(SkullVisAtt);

    G4cout << "Skull created !!!!!!" << G4endl;

    // Testing Skull Volume
    G4double SkullVol = logicSkull->GetSolid()->GetCubicVolume();
    G4cout << "Volume of Skull = " << SkullVol / cm3 << " cm^3" << G4endl;

    // Testing Skull Material
    G4String SkullMat = logicSkull->GetMaterial()->GetName();
    G4cout << "Material of Skull = " << SkullMat << G4endl;

    // Testing Density
    G4double SkullDensity = logicSkull->GetMaterial()->GetDensity();
    G4cout << "Density of Material = " << SkullDensity * cm3 / g << " g/cm^3" << G4endl;

    // Testing Mass
    G4double SkullMass = (SkullVol)*SkullDensity;
    G4cout << "Mass of Skull = " << SkullMass / gram << " g" << G4endl;

    return physSkull;
}
