#include "G4HumanPhantomColour.h"
#include "G4HumanPhantomMaterial.h"
#include "G4MIRDUterus.h"

#include "G4Ellipsoid.hh"
#include "G4LogicalVolume.hh"
#include "G4Material.hh"
#include "G4PVPlacement.hh"
#include "G4RotationMatrix.hh"
#include "G4SDManager.hh"
#include "G4SystemOfUnits.hh"
#include "G4ThreeVector.hh"
#include "G4VPhysicalVolume.hh"
#include "G4VisAttributes.hh"
#include "globals.hh"

G4MIRDUterus::G4MIRDUterus() {}

G4MIRDUterus::~G4MIRDUterus() {}

G4VPhysicalVolume* G4MIRDUterus::Construct(const G4String& volumeName, G4VPhysicalVolume* mother,
                                           const G4String& colourName, G4bool wireFrame, G4bool) {
    G4cout << "Construct " << volumeName << " with mother volume " << mother->GetName() << G4endl;

    G4HumanPhantomMaterial* material = new G4HumanPhantomMaterial();
    G4Material* soft = material->GetMaterial("soft_tissue");
    delete material;

    G4double ax = 2.5 * cm;  // a
    G4double by = 1.5 * cm;  // c
    G4double cz = 5. * cm;   // b

    G4double zcut1 = -5. * cm;  //-b
    G4double zcut2 = 2.5 * cm;  // y1-y0

    G4Ellipsoid* uterus = new G4Ellipsoid("Uterus", ax, by, cz, zcut1, zcut2);

    G4LogicalVolume* logicUterus = new G4LogicalVolume(uterus, soft, "logical" + volumeName, 0, 0, 0);

    // Define rotation and position here!
    G4RotationMatrix* rm = new G4RotationMatrix();
    rm->rotateX(90. * degree);
    G4VPhysicalVolume* physUterus = new G4PVPlacement(rm, G4ThreeVector(0. * cm, 2 * cm, -21 * cm),
                                                      "physicalUterus",  // y0
                                                      logicUterus, mother, false, 0, true);

    // Visualization Attributes
    //  G4VisAttributes* UterusVisAtt = new G4VisAttributes(G4Colour(0.85,0.44,0.84));

    G4HumanPhantomColour* colourPointer = new G4HumanPhantomColour();
    G4Colour colour = colourPointer->GetColour(colourName);
    G4VisAttributes* UterusVisAtt = new G4VisAttributes(colour);
    UterusVisAtt->SetForceSolid(wireFrame);
    logicUterus->SetVisAttributes(UterusVisAtt);

    G4cout << "Uterus created !!!!!!" << G4endl;

    // Testing Uterus Volume
    G4double UterusVol = logicUterus->GetSolid()->GetCubicVolume();
    G4cout << "Volume of Uterus = " << UterusVol / cm3 << " cm^3" << G4endl;

    // Testing Uterus Material
    G4String UterusMat = logicUterus->GetMaterial()->GetName();
    G4cout << "Material of Uterus = " << UterusMat << G4endl;

    // Testing Density
    G4double UterusDensity = logicUterus->GetMaterial()->GetDensity();
    G4cout << "Density of Material = " << UterusDensity * cm3 / g << " g/cm^3" << G4endl;

    // Testing Mass
    G4double UterusMass = (UterusVol)*UterusDensity;
    G4cout << "Mass of Uterus = " << UterusMass / gram << " g" << G4endl;

    return physUterus;
}
